DrawSprites:
    ld hl,SpriteTable
    ld b,SpriteTotal
DrawSpritesLoop:
    ;Get Sprite
    push bc
    xor a
    ld b,(hl)
    inc hl
    ld c,(hl)
    inc hl
    push hl
    ld l,(hl)
    ld h,a
    or l
    jp z,SkipDisp
    cp $80
    jp nc,SkipDisp
    add hl,hl   ;mult by 8
    add hl,hl
    add hl,hl
    ld de,Player
    add hl,de
    call PutSprClpAND
SkipDisp:
    pop hl
    ld bc,6
    add hl,bc
    pop bc
    djnz DrawSpritesLoop
    ret
    
DrawMap:
    ld de, level
    ld hl, _plotSScreen ;$FC00
    ld bc, 16*256+0
DrawMapLoop:
    ex de,hl
    push de
    xor a
    ld e,(hl)
    or e
    jp z,DrawMapSkip1
    ld e,2
DrawMapSkip1:
    inc hl
    ld a,(hl)
    or a
    jp z,DrawMapSkip2
    ld a,1
DrawMapSkip2:
    or e
    pop de
    ex de,hl
    push de
    push hl
    ld l, a
    ld h, 0
    add hl, hl
    add hl, hl
    ld de, Tile
    add hl, de
    ex de, hl
    pop hl
    push bc
    ld b, 4
    push hl
DrawmapLoop2:
    ld a, (de)
    ld (hl),a
    inc de
    push bc
    ld bc, 16
    add hl, bc
    pop bc
    djnz DrawmapLoop2
    pop hl
    pop bc
    pop de
    inc de
    inc hl
    inc c
    ld a, c
    cp 12
    jr nz, DrawMapLoop
    ld c,0
    push de
    ld de, 3*16+4
    add hl, de
    pop de
    djnz DrawMapLoop
    ret

SwapAPD:
    ld hl,_plotSScreen
    ld de,$AF80             ;safemem1
    ld bc,1024
SwapMem:
    ld a,(de)
    ldi
    dec hl
    ld (hl),a
    inc hl
    jp pe,SwapMem
    ret
Copy2APD:
    ld bc,1024
    ld de,$AF80
    ld hl,_plotSScreen
    ldir
    ret
CopyAPD:
    ld bc,1024
    ld de,_plotSScreen
    ld hl,$AF80
    ldir
    ret
BlackScreen:
    ld hl,_plotSScreen
    ld de,_plotSScreen+1
    ld bc,1023
    ld (hl),$FF
    ldir
    ret


PowerOff:
;  ld a, 2
;  out ($10), a          ; display off
;  ld a, 1
;  out (3), a            ; enable only [on] int, and turn lcd driver into standby mode
;    ei
;  halt                  ; wait 'til [on] keypress
;  di
;  ld a, %1011
;  out (3), a            ; enable [on] int, and periodic int. Turn on LCD driver
;  ld a, 3
;  out ($10), a          ; display on
;  di
  ret

ScrollUp:
    call IonFastCopy
    ld b,64
    ld hl,0
ScrollUpLoop:
    push bc
    push hl
    ld de,_plotSScreen
    ld hl,_plotSScreen+16
    ld bc,1008
    ldir
    pop hl
    push hl
    ld bc,$AF80             ;saferam1
    add hl,bc
    ld de,_plotSScreen+1008
    ld bc,16
    ldir
    pop hl
    ld bc,16
    add hl,bc
    push hl
    call IonFastCopy
    pop hl
    pop bc
    djnz ScrollUpLoop
    ret

DispRLE:
 ld bc,1024         ; we need to copy 
DispRLEL:
 ld a,(hl)          ; get the next byte
 cp $91             ; is it a run?
 jr z,DispRLERun    ; then we need to decode the run
 ldi                ; copy the byte, and update counters
DispRLEC:
 ld a,b             ; check the low byte and
 or c               ; the high byte for 0
 jr nz,DispRLEL     ; if not, then we're not done either
 ret                ; if it's zero, we're done
DispRLERun:
 inc hl             ; move to the run value
 ld a,(hl)          ; get the run value
 inc hl             ; move to the run count
 push hl            ; save source pointer
 ld h,(hl)          ; get the run count
 ex de,hl           ; swap source and destination pointers
DispRLERunL:
 ld (hl),a          ; copy the byte
 inc hl             ; increase destination pointer
 dec bc             ; decrease byte count
 dec d              ; decrease run count
 jr nz,DispRLERunL  ; if we're not done, then loop
 ex de,hl           ; swap pointers back
 pop hl             ; recover source pointer
 inc hl             ; advance the source pointer
 jr DispRLEC        ; check to see if we should loop
 
 
;==========================================================
;   Stretch:  Stretches an 8x8 sprite to a 16x16 sprite
;==========================================================
;Input: (hl)=sprite to stretch   (de)=output (32 bytes)
;Column aligned version
Stretch:
    ld de,BigSprite
    ld b,8          ;8 Rows to stretch
StretchMainLoop:    ;Begin a loop
    push bc         ;Save row count
    ld c,(hl)       ;Get data from sprite
    push hl                 ;Save source offset
    push de                 ;Save destination offset
    ld h,%10000000  ;Load in the masks, first one is for source pixel
    ld l,%11000000  ;Second one is for destination pixel, this is a wide pixel
    call StretchIt  ;Stretch the left half
    ld e,d                  ;Save the result byte
    call StretchIt  ;Stretch the right half
    pop hl                  ;Get destination offset
    ld bc,15        ;Set addition offset  
    ld (hl),e               ;Write 2 new bytes...
    inc hl
    ld (hl),e
    add hl,bc       ;Add 15 to offset...
    ld (hl),d       ;Write 2 new bytes...
    inc hl
    ld (hl),d 
    sbc hl,bc
    ex de,hl                ;Restore destination back to de
    pop hl                  ;Get source offset
    inc hl                  ;Increment source offset
    pop bc                  ;Get row count
    djnz StretchMainLoop    ;Loop back 8 times
    ret                         ;Return
StretchIt:
  ld d,0                    ;Clear the destination byte
  ld b,4                    ;4 big pixels
StretchLoop:            ;Begin a loop
    ld a,c                  ;Check the pixel by anding the
    and h                       ; source byte with the pixel mask
    jp z,St_NoPixel ;Skip ahead if no pixel
    ld a,d                  ;Set the new big pixel
    or l                        ; by oring the new byte with
    ld d,a                  ; the destination pixel mask
St_NoPixel:             ;
    rrc h                       ;Shift the masks around for the next pixel
    rrc l                       ;
    rrc l                       ;
    djnz StretchLoop;Loop 4 times
    ret                         ;Return


;========================================================================
;  Disp16Sprite  - Draws a 16x16 AND sprite using crashman's routine
;========================================================================
Disp16Sprite:
    ld a,16
    call SetHeightAND
    ld bc,(BigspriteY)
    ld hl,BigSprite
    call PutSprClpAND
    ld hl,(BigspriteY)
    ld bc,$0800
    add hl,bc
    push hl
    pop bc
    ld hl,BigSprite+16
    call PutSprClpAND
    ld a,8
    jp SetHeightAND

;=================================================
;  RepSpr - Draws an aligned 8x8 replace sprite
;=================================================
RepSpr:
    ld (RepSprModify+1),a
    push hl
    ld a,b
    and %01111000
    rra
    rra
    rra
    ld e,a
    xor a
    ld d,a
    ld b,a
    ld h,a
    ld l,c
    add hl,hl
    add hl,hl
    add hl,hl
    add hl,hl
    
;    add hl,bc   ;*2
;    add hl,bc   ;*3
;    add hl,hl   ;*6
;    add hl,hl   ;*12
    add hl,de   ;+x
    ld de,_plotSScreen
    add hl,de
RepSprModify:
    ld b,8
    ex de,hl
    pop hl
RepSprLoop:
    push bc
    ldi
    ld bc,15
    ex de,hl
    add hl,bc
    ex de,hl
    pop bc
    djnz RepSprLoop
    ret

;==========================================
;  VDispA - Displays A in the small font
;==========================================
vDispA:
    ld h,0
    ld l,a
;===========================================
;  VDispHL - Displays hl in the small font
;===========================================
vDispHL:
    push de
    push hl
    ld de,_OP1+5
    xor a
    ld (de),a
    ld b,a          ;;;;;;;;
vdhlRepeat:
    call _divHLby10
    add a,'0'
    dec de
    ld (de),a
    inc b
    ld a,h
    or l
    jr nz,vdhlRepeat
    ex de,hl
    ld a,b
    add a,a
    add a,a
    ld b,a
    ld a,(_penCol)
    sub b
    ld (_penCol),a
    call _vputs
    pop hl
    pop de
    ret
